from enum import Enum
from typing import List, Optional, Dict

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType


class MediaLibraryPath:
    path: str
    auto_scan: bool = False

    def __init__(self, data: Dict = None):
        if not data:
            utils.copy_value(data, self)


class TransferMode(Enum):
    HardLink = 'link'
    Copy = 'copy'
    Move = 'move'


class LibraryApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def start_scanner(self, library_id: int):
        self._session.get('library.start_scanner', {
            'library_id': library_id
        })

    def stop_scanner(self, library_id: int):
        self._session.get('library.stop_scanner', {
            'library_id': library_id
        })

    def add_library(self, media_type: MediaType, library_name: str, library_paths: List[MediaLibraryPath]) -> Optional[
        int]:
        return self._session.post('library.add_library', {
            'library_name': library_name,
            'media_type': media_type.value,
            'library_paths': [utils.to_dict(x) for x in library_paths]
        })

    def rename_by_path(self, path: str):
        return self._session.post('library.rename_by_path', {
            'path': path
        })

    def direct_transfer(self, src: str, dst: str, mode: TransferMode):
        self._session.post('library.direct_transfer', {
            'src': src,
            'dst': dst,
            'mode': mode.value
        })



from typing import Dict

from moviebotapi import Session
from moviebotapi.core import utils


class DoubanConfig:
    cookie: str

    def __init__(self, data: Dict, session: Session):
        utils.copy_value(data, self)
        self._ = session

    def save(self):
        """
        保存豆瓣配置，保存过程服务端会检查cookie的有效性，不是简单的保存配置文件
        """
        self._.post('setting.save_douban', json={
            'cookie': self.cookie
        })


class FreeDownloadConfig:
    available_space: int
    avg_statistics_period: int
    enable: bool
    maximum_active_torrent: int
    save_path: str
    upload_mbps_maximum: int

    def __init__(self, data: Dict, session: Session):
        utils.copy_value(data, self)
        self._ = session

    def save(self):
        self._.post('setting.save_free_download', json={
            'enable': self.enable,
            'save_path': self.save_path,
            'available_space': self.available_space,
            'avg_statistics_period': self.avg_statistics_period,
            'upload_mbps_maximum': self.upload_mbps_maximum,
            'maximum_active_torrent': self.maximum_active_torrent
        })


class WebConfig:
    """web访问配置"""
    host: str
    port: int
    # 外网访问地址
    server_url: str

    def __init__(self, data: Dict, session: Session):
        utils.copy_value(data, self)
        self._ = session

    def save(self):
        self._.post('setting.save_web', json={
            'host': self.host,
            'port': self.port,
            'server_url': self.server_url
        })


class Env:
    config_dir: str
    user_config_dir: str
    site_config_dir: str
    plugin_dir: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class ConfigApi:
    def __init__(self, session: Session):
        self._session: Session = session

    @property
    def douban(self):
        return DoubanConfig(self._session.get('setting.get_douban'), self._session)

    @property
    def free_download(self):
        return FreeDownloadConfig(self._session.get('setting.get_free_download'), self._session)

    @property
    def web(self):
        return WebConfig(self._session.get('setting.get_web'), self._session)

    @property
    def env(self):
        return Env(self._session.get('setting.get_env'))



from typing import Dict

from moviebotapi import Session


class AuthApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get_default_ak(self, ) -> Dict:
        return self._session.get('auth.get_default_ak')



from typing import Union, Optional, List

from moviebotapi.core import utils
from moviebotapi.core.models import MediaType
from moviebotapi.douban import DoubanMedia
from moviebotapi.tmdb import TmdbMovie, TmdbTV


class MediaMetaSelect:
    def __init__(self, douban: Optional[DoubanMedia] = None, tmdb: Optional[Union[TmdbMovie, TmdbTV]] = None):
        self.douban = douban
        self.tmdb = tmdb

    @property
    def country(self):
        country: List[str] = list()
        if self.tmdb:
            if hasattr(self.tmdb, 'production_countries') and not self.tmdb.production_countries and hasattr(self.tmdb,
                                                                                                             'origin_country') and not self.tmdb.origin_country:
                return ['其他']
            if hasattr(self.tmdb, 'origin_country') and self.tmdb.origin_country:
                for c in self.tmdb.origin_country:
                    country.append(utils.Countries.get(c))
            if len(country) == 0 and hasattr(self.tmdb, 'production_countries') and self.tmdb.production_countries:
                for c in self.tmdb.production_countries:
                    country.append(utils.Countries.get(c.get('iso_3166_1')))
        if not country and self.douban:
            country = self.douban.country
        return country

    @property
    def genres(self):
        genres = None
        if not genres and self.tmdb and self.tmdb.genres:
            arr = []
            for item in self.tmdb.genres:
                if item.name == 'Sci-Fi & Fantasy':
                    arr.append('科幻')
                elif item.name == 'War & Politics':
                    arr.append('战争')
                else:
                    arr.append(item.name)
            genres = arr
        if not genres and self.douban:
            genres = self.douban.genres
        return genres

    @property
    def episode_count(self):
        if self.douban and self.douban.episode_count:
            return self.douban.episode_count
        if self.tmdb and isinstance(self.tmdb, TmdbTV):
            return self.tmdb.number_of_episodes
        return 1

    @property
    def title(self):
        title = None
        if self.douban:
            if self.douban.media_type == MediaType.TV:
                title = utils.MediaParser.trim_season(self.douban.cn_name)
            else:
                title = self.douban.cn_name
            if title == '未知电视剧' or title == '未知电影':
                title = None
        if self.tmdb and not title:
            if isinstance(self.tmdb, TmdbMovie):
                title = self.tmdb.title
            elif isinstance(self.tmdb, TmdbTV):
                title = self.tmdb.name
        return title

    @property
    def rating(self):
        rating = None
        if self.douban:
            rating = self.douban.rating
        if not rating and self.tmdb:
            rating = self.tmdb.vote_average
        if rating:
            return round(rating, 1)
        else:
            return rating

    @property
    def url(self):
        if self.douban:
            return 'https://movie.douban.com/subject/%s/' % self.douban.id
        if self.tmdb:
            if isinstance(self.tmdb, TmdbMovie):
                return 'https://themoviedb.org/movie/%s' % self.tmdb.id
            else:
                return 'https://themoviedb.org/tv/%s' % self.tmdb.id
        return

    @property
    def intro(self):
        intro = None
        if self.douban:
            intro = self.douban.intro
        if self.tmdb and not intro:
            intro = self.tmdb.overview
        if intro:
            return str(intro).strip()
        else:
            return intro

    @staticmethod
    def _get_tmdb_tv_date(tv: TmdbTV) -> Optional[str]:
        if tv.first_air_date:
            return tv.first_air_date
        elif tv and tv.seasons and tv.seasons[0].air_date:
            return tv.seasons[0].air_date[0:4]
        return

    @property
    def release_year(self):
        if self.douban:
            return self.douban.release_year
        if self.tmdb:
            if isinstance(self.tmdb, TmdbMovie):
                if self.tmdb.release_date:
                    return self.tmdb.release_date[0:4]
                else:
                    return
            else:
                tv_data = self._get_tmdb_tv_date(self.tmdb)
                return tv_data[0:4] if tv_data else None

    @property
    def release_date(self):
        date = None
        if self.tmdb:
            if isinstance(self.tmdb, TmdbMovie):
                if self.tmdb.release_date:
                    date = self.tmdb.release_date
            else:
                date = self._get_tmdb_tv_date(self.tmdb)
        if self.douban and not date:
            date = self.douban.premiere_date
        return date
